//
// Fade-in LED driver
// ATtiny402
// MaiaR create
//

#include <tinyNeoPixel_Static.h>

#define PIN  0  // to NeoPixel LED
#define NLED 10 // number of NeoPixel LEDs

#define BRIGHTRGBMAX  120 // Process of increase for NeoPixel
#define BRIGHTRGBMAX2 (BRIGHTRGBMAX * BRIGHTRGBMAX)
#define BRIGHTMAX  50 // Process of increase for LED
#define BRIGHTMAX2 (BRIGHTMAX * BRIGHTMAX)

#define DELAYVAL  20 // step time 20ms
#define INTERVAL0 50 // 50 x 20ms = 1s     start up time 1st LED
#define INTERVAL1 25 // 25 x 20ms = 0.5s   2nd LED
#define INTERVAL2 22 // 22 x 20ms = 0.44s  3rd LED
#define INTERVAL3 24 // 24 x 20ms = 0.48s  4th LED

uint8_t pixels[NLED * 3];
int16_t brightrgb;
uint16_t bright[4];
int16_t starttime[4];

tinyNeoPixel ledbar = tinyNeoPixel(NLED, PIN, NEO_GRB, pixels);

struct pix{
  unsigned char g; // green
  unsigned char r; // red
  unsigned char b; // blue
};

struct pix *planes;
uint8_t shift = 0;
uint8_t speed = 1;

#define BOT 0
#define TOP 255
pix WheelB(uint8_t WheelPos){
  pix result;
  if(WheelPos < 85) {
    WheelPos *= 3;
    result.r = TOP - WheelPos;
    result.g = BOT + WheelPos;
    result.b = BOT;
  } else if(WheelPos < 170) {
    WheelPos -= 85;
    WheelPos *= 3;
    result.r = BOT;
    result.g = TOP - WheelPos;
    result.b = BOT + WheelPos;
  } else {
    WheelPos -= 170;
    WheelPos *= 3;
    result.r = BOT + WheelPos;
    result.g = BOT;
    result.b = TOP - WheelPos;
  }
  return (result);
}

void setplanes(uint16_t y, pix data, uint8_t br){
  planes[y].r = data.r * br / 255;
  planes[y].g = data.g * br / 255;
  planes[y].b = data.b * br / 255;
}


void setup() {
  pinModeFast(PIN, OUTPUT);
  ledbar.clear();
  planes = (pix*)ledbar.getPixels();
  ledbar.show();
  
  starttime[0] = INTERVAL0;
  starttime[1] = starttime[0] + INTERVAL1;
  starttime[2] = starttime[1] + INTERVAL2;
  starttime[3] = starttime[2] + INTERVAL3;
}

void loop() {
  static int16_t tick = 0;

  //------------ mono color LED ------------------------------
  for (uint8_t i = 0; i < 4; i++){
    bright[i] = constrain(tick - starttime[i], 0, BRIGHTMAX);
    analogWrite(i + 1, map(bright[i] * bright[i], 0, BRIGHTMAX2, 0, 255));
  }
  //----------------------------------------------------------

  //------------- full color LED -----------------------------
  brightrgb = constrain(tick - INTERVAL0, 0, BRIGHTRGBMAX);
  for(uint8_t i = 0; i < NLED; i++){
    setplanes(i, WheelB(i * 256 / NLED + shift), map(brightrgb * brightrgb, 0, BRIGHTRGBMAX2, 0, 255));
  }
  shift += speed;
  ledbar.show();
  //----------------------------------------------------------

  if (tick <= (starttime[3] + BRIGHTMAX) || tick <= (INTERVAL0 + BRIGHTRGBMAX)){
    tick++;
  }
  
  delay(DELAYVAL);
}
